package org.jorgecardoso.net;

import java.io.*;


/**
 * Utility class for HTML form encoding. This class contains static methods
 * for converting a String to the <CODE>application/x-www-form-urlencoded</CODE> MIME
 * format. For more information about HTML form encoding, consult the HTML
 * <A HREF="http://www.w3.org/TR/html4/">specification</A>.
 *
 * <p>
 * When encoding a String, the following rules apply:
 *
 * <p>
 * <ul>
 * <li>The alphanumeric characters &quot;<code>a</code>&quot; through
 *     &quot;<code>z</code>&quot;, &quot;<code>A</code>&quot; through
 *     &quot;<code>Z</code>&quot; and &quot;<code>0</code>&quot;
 *     through &quot;<code>9</code>&quot; remain the same.
 * <li>The special characters &quot;<code>.</code>&quot;,
 *     &quot;<code>-</code>&quot;, &quot;<code>*</code>&quot;, and
 *     &quot;<code>_</code>&quot; remain the same.
 * <li>The space character &quot;<code>&nbsp;</code>&quot; is
 *     converted into a plus sign &quot;<code>+</code>&quot;.
 * <li>All other characters are unsafe and are first converted into
 *     one or more bytes using some encoding scheme. Then each byte is
 *     represented by the 3-character string
 *     &quot;<code>%<i>xy</i></code>&quot;, where <i>xy</i> is the
 *     two-digit hexadecimal representation of the byte.
 *     The recommended encoding scheme to use is UTF-8. However,
 *     for compatibility reasons, if an encoding is not specified,
 *     then the default encoding of the platform is used.
 * </ul>
 *
 * <p>
 * @author  Jorge Cardoso
 * @version 1.00, 16/07/2005
 */
public class URLEncoder {

	/**
	 * Encodes an already formed URL using the <CODE>application/x-www-form-urlencoded</CODE>
	 * MIME format.
	 *
	 * @param s The String (URL) to be converted.
	 *
	 * @return The converted String (URL).
	 */
	public static String URLEncode(String s) {
		int questionMark = s.indexOf('?');

		if (questionMark == -1) { // no parameters
			return s;
		} else {
			return s.substring(0, questionMark+1) + encodeURL(s.substring(questionMark+1));
		}
	}

	/**
	 * Base on code in http://forums.java.sun.com/thread.jspa?threadID=409913&messageID=1806947
	 * Converts a String to the <CODE>application/x-www-form-urlencoded</CODE> MIME
  	 * format.
  	 *
  	 * @param s The String to be converted.
  	 *
  	 * @return The converted String.
	 */
	public static String encode(String s) {
		StringBuffer out = new StringBuffer();
		ByteArrayOutputStream bOut = new ByteArrayOutputStream();

		DataOutputStream dOut = new DataOutputStream(bOut);

		try {
			dOut.writeUTF(s);
		} catch (IOException ioe) {
			return null;
		}

		ByteArrayInputStream bIn = new ByteArrayInputStream(bOut.toByteArray());

		/* UTF strings have 2 bytes to indicate the size, so skip these	*/
		bIn.read();
		bIn.read();

		int c = bIn.read();
		while (c >= 0) {
			if ((c >= 'a' && c <= 'z')
					|| (c >= 'A' && c <= 'Z')
					|| (c >= '0' && c <= '9')
					|| c == '.' || c == '-' || c == '*' || c == '_') {
				out.append((char) c);
			} else if (c == ' ') {
				out.append('+');
			} else {
				if (c < 128) {
					appendHex(c,out);
				} else if (c < 224) {
					appendHex(c,out);
					appendHex(bIn.read(),out);
				} else if (c < 240) {
					appendHex(c,out);
					appendHex(bIn.read(),out);
					appendHex(bIn.read(),out);
				}

			}
			c = bIn.read();
		}
		return out.toString();
	}

	/**
	 * Base on code in http://forums.java.sun.com/thread.jspa?threadID=409913&messageID=1806947
	 *
	 * Similar to encode but we don't encode the '=' and '&' characters. This
	 * way we can use this to encode an already formed URL (or better, the parameter
	 * part of an already formed URL).
	 */
	private static String encodeURL(String s) {
		StringBuffer out = new StringBuffer();
		ByteArrayOutputStream bOut = new ByteArrayOutputStream();

		DataOutputStream dOut = new DataOutputStream(bOut);

		try {
			dOut.writeUTF(s);
		} catch (IOException ioe) {
			return null;
		}

		ByteArrayInputStream bIn = new ByteArrayInputStream(bOut.toByteArray());

		/* UTF strings have 2 bytes to indicate the size, so skip these	*/
		bIn.read();
		bIn.read();

		int c = bIn.read();
		while (c >= 0) {
			if ((c >= 'a' && c <= 'z')
					|| (c >= 'A' && c <= 'Z')
					|| (c >= '0' && c <= '9')
					|| c == '.' || c == '-'
					|| c == '*' || c == '_'
					|| c == '=' || c == '&') {
				out.append((char) c);
			} else if (c == ' ') {
				out.append('+');
			} else {
				if (c < 128) {
					appendHex(c,out);
				} else if (c < 224) {
					appendHex(c,out);
					appendHex(bIn.read(),out);
				} else if (c < 240) {
					appendHex(c,out);
					appendHex(bIn.read(),out);
					appendHex(bIn.read(),out);
				}

			}
			c = bIn.read();
		}
		return out.toString();
	}

	private static void appendHex(int arg0, StringBuffer buff){
		buff.append('%');
		if (arg0<16) {
			buff.append('0');
		}
		buff.append(Integer.toHexString(arg0));
	}
}